% F = fonction qui renvoie F et DF en fonction de X (standard des fonctions
% d'optim de matlab) et eventuellement HF si l'option idoine est sur 'on'
% X = point d'evaluation
% norme = 1, 2, 'inf'  (un type de norme reconnu par matlab)
% options.DerivativeCheck = 'on', 'off' (pour coller au fonctionnement des
% fonctions d'optim de matlab, sans quoi j'aurais prfr un booleen...)
% options.Hessian = 'on', 'off'
function VerificationDifferentielle( F, X, norme, options )
if isfield( options, 'DerivativeCheck') && strcmp( options.DerivativeCheck, 'on' )
    [ FX, DFX ] = F(X);
    if size( DFX, 2)==1, DFX = DFX'; end
    
    H = rand( size( X ) );
    puissances = 2:12;
    ecarts = ones( max( puissances ), 1 );
    
    for iEpsilon = puissances
        E = 10^(-iEpsilon);
        ecarts(iEpsilon) =  norm( (F(X+E*H)-F(X-E*H))/(2*E) - DFX*H, norme )...
                            /(1+norm( DFX*H, norme ));
    end
    
    [ minEcart, iEcart ] = min( ecarts );
    if minEcart > 1e-6
        disp( [ 'Approximation relative 1er ordre : ',...
            num2str( minEcart, '%E' ), ' en E=10-', num2str( iEcart ) ] );
    end
    if options.dessins
        figure('visible', 'on'); semilogy(ecarts);
        title( 'Difference relative accroissements finis et derivee' );
        %pause;
    end
    
%     if isfield( options, 'Hessian') && strcmp( options.Hessian, 'on' )
%         [ DFX, HFX ] = DF( F, X);
%         for iEpsilon = puissances
%             E = 10^(-iEpsilon);
%             ecarts(iEpsilon) =  norm( (DF( F, X+E*H)-DF( F, X-E*H))/(2*E) - HFX*H, norme )...
%                 /(1+norm( HFX*H, norme ));
%         end
%         
%         [ minEcart, iEcart ] = min( ecarts );
%         disp( [ 'inf(DF(x+Eh)-DF(x-Eh))/2E-HFx*h =  ',...
%             num2str( minEcart, '%E' ), ' en E=10-', num2str( iEcart ) ] );
%         figure('visible', 'on'); semilogy(ecarts); 
%         title( 'Difference relative accroissements finis de la derivee et hessienne' );
%         pause;
%     end
    if isfield( options, 'Hessian') && strcmp( options.Hessian, 'on' )
        [ FX, GFX, HFX ] = F(X);
        
        H = rand( size( X ) );
        puissances = 2:12;
        ecarts = ones( max( puissances ), 1 );
        
        for iEpsilon = puissances
            E = 10^(-iEpsilon);
            ecarts(iEpsilon) =  norm( (F(X+E*H)+F(X-E*H)-2*FX)/(E*E) - H'*HFX*H, norme )...
                /norm( 1+H'*HFX*H, norme );
        end
        
        [ minEcart, iEcart ] = min( ecarts );
        if minEcart > 1e-6
            disp( [ 'Approximation relative 2nd ordre : ',...
                num2str( minEcart, '%E' ), ' en E=10-', num2str( iEcart ) ] );
        end
        if options.dessins
            figure('visible', 'on'); semilogy(ecarts);
            title( 'Difference relative entre accroissements finis et 2nd ordre' );
            %pause;
        end
    end
    
end

end

function [ DFX, HFX ] = DF( F, X )
[ ~, DFX, HFX ] = F( X );
end