% minimisation de f_W par differentes methodes
% options.methode = 'descente matlab', 'descente', 'projection',
% 'contrainte matlab'
function [carte,data] = indicatrice(u_0, physique, F1, F0, options)
tic

nPoints = size(u_0,1);
nEmissions = size(F1,2);
ID = speye(2*nEmissions);
carte = zeros(nPoints,1);
data.xStart = zeros(2*nEmissions,nPoints);
data.xFinal = zeros(2*nEmissions,nPoints);
data.grad = zeros(2*nEmissions,nPoints);
data.iterations = zeros(nPoints,1);

[W, demiMasse] = operateur_mesures( F0, F1, physique.nombreOnde, physique.anglesEmissions, physique.anglesMesures, options.operateur);%options.operateur

WR = extension((W+W')/2);
WI = extension((W-W')/2i);
W = extension(W);

optionsOptimisation = optimset;
optionsOptimisation.MaxIter = 400;
optionsOptimisation.TolFun = 1e-52; %options.tolFun; % ok : 1e-32
optionsOptimisation.TolX = 1e-9; %options.tolX; % ok : 1e-9
optionsOptimisation.Display = 'off';%'final-detailed';%
optionsOptimisation.GradObj = 'on';
optionsOptimisation.Hessian = 'on';
optionsOptimisation.DerivativeCheck = 'off';
%optionsOptimisation.Diagnostics = 'on';
%optionsOptimisation.Algorithm = 'active-set';
%optionsOptimisation.LargeScale = 'off';
optionsOptimisation.UseParallel = 'always';

for iPoints = 1:nPoints
    ubar = demiMasse*u_0(iPoints,:)';
    u2 = ubar'*ubar; % abs(ubar'*masse*ubar);
    pFleche = ID - extension(ubar*ubar'/u2);
    X0 = extension(ubar/u2);
    data.xStart(:,iPoints) = X0;
    offset = extension(ubar/u2);
    switch options.methode 
        case 'descente matlab'
            [X0, Pf4X, ~, output, direction] = fminunc( @(X) Pf(X, W, WR, WI,...
                pFleche, offset, options.operateur) ,X0 ,optionsOptimisation);
            data.iterations(iPoints) = output.iterations;
        case 'descente'
            for iIterations = 1:optionsOptimisation.MaxIter
                VerificationDifferentielle( @(X) Pf(X, W, WR, WI,...
                        pFleche, offset, options.operateur) ,X0 , 'inf', optionsOptimisation );
                [Pf4X, gradPf4, ~] = Pf(X0, W, WR, WI, pFleche, offset, options.operateur);
                direction = -gradPf4;
                alpha = line_search(@(X) Pf(X, W, WR, WI, pFleche, offset, ...
                options.operateur), X0, 1, direction, 'dichotomie');
                X1 = X0 + alpha*direction;
                if norm(X1-X0)/(1+norm(X0)) < optionsOptimisation.TolX, break, end
                %if norm(direction) < optionsOptimisation.TolFun, break, end
                X0 = X1;
            end
            data.iterations(iPoints) = iIterations;
        case 'projection'
            for iIterations = 1:optionsOptimisation.MaxIter
                VerificationDifferentielle( @(X) Pf(X, W, WR, WI,...
                        ID, 0, options.operateur) ,X0 , 'inf', optionsOptimisation );
                [Pf4X, gradPf4, ~] = Pf(X0, W, WR, WI, ID, 0, options.operateur);
                direction = -gradPf4;
                alpha = line_search(@(X) Pf(X, W, WR, WI, ID, 0, ...
                options.operateur), X0, 1, direction, 'dichotomie');
                X1 = pFleche*(X0 + alpha*direction) + offset;
                if norm(X1-X0)/(1+norm(X0)) < optionsOptimisation.TolX, break, end
                %if norm(direction) < optionsOptimisation.TolFun, break, end
                X0 = X1;
            end
            data.iterations(iPoints) = iIterations;
        case 'contrainte matlab'
            [X0, Pf4X, ~, output, ~, direction] = fmincon(@(X) Pf(X, W, WR, WI,...
                ID, 0, options.operateur) ,X0 , [], [], extension(ubar)'...
                , 1, [], [], [], optionsOptimisation);
            data.iterations(iPoints) = output.iterations;
        case 'test'
            valeurs = zeros(nEmissions,1);
            X = spdiags(1./conj(ubar), 0, nEmissions, nEmissions);
            for iEmission=1:nEmissions
                X0 = extension(X(:,iEmission));
                valeurs(iEmission) = Pf(X0, W, WR, WI, ID, 0, options.operateur);
            end
            [Pf4X, iEmission] = min(valeurs);
            X0 = extension(X(:,iEmission));
            direction = 0;
     end
     
    carte(iPoints) = Pf4X;
    data.xFinal(:,iPoints) = X0;
    data.grad(:,iPoints) = direction;
end
data.time = toc;
end

% Pf4x = P_{f^4_W}(x) = f^4(Px) et gradPf4 = grad P_{f^4_W}(x)
function [Pf4x,gradPf4,hessianPf4] = Pf(X, W, WR, WI, pFleche, offset ,operateur)
switch operateur
    case 'W'
        Px = pFleche*X+offset; % on projete x en y
        WRPx = WR*Px;
        WIPx = WI*Px;
        WRPxPx = Px'*WRPx; %partie reelle de Wy,y
        WIPxPx = Px'*WIPx;
        Pf4x = WRPxPx^2 + WIPxPx^2;
        if nargout>1 % note
            gradPf4 = 4*pFleche'*(WRPxPx*WRPx + WIPxPx*WIPx);
            if nargout>2
                hessianPf4 = 4*pFleche'*( ...
                    2*(WRPx*WRPx') + WRPxPx*WR + 2*(WIPx*WIPx') + WIPxPx*WI ...
                    )*pFleche;
            end
        end
    case {'|W|','W#'}
        Px = pFleche*X+offset; % on projete x en y
        WPx = W*Px;
        Pf4x = Px'*WPx;
        if nargout>1
            gradPf4 = 2*pFleche'*WPx;
            if nargout>2
                hessianPf4 = 2*pFleche'*W*pFleche;
            end
        end
end
end




