%
% version modifiee pour la geometrie 'cercle_de_reference' afin de
% transmettre les rayons
%
%[p,t,e,ebo,zone_ext,lpo,xmax,xmin,ymax,ymin,mplo_moyen]=maillage_diffracteur(dessin,lambda_par_zone,jiggle,mailles_par_longueur_d_onde,nombre_d_onde,fichier_geometrie,indices)
%cree un maillage a partir d'un fichier de geometrie qui a toutes les
%donnees - 
%les  entrees 3 et suivantes sont les indices des milieux a
%indicer prevus par le fichier geometrie
%
% entrees :
%   dessin (0/1) : dessine la geometrie 
%   lambda par zone : nombre de longeurs d'onde entre l'objet et
%                     l'observation, ainsi qu'entre l'observation et le debut des pml
%   jiggle : conserve uniquement pour compatibilite d'interface
% sorties :
%   ebo : elements du bord d'observation (extrait de e)
%   zone_ext : conserve uniquement pour compatibilite d'interface
%   lpo : liste des points de bord de l'objet (extrait de p)
%   xmin max ... : valeurs de debut de pml (a destination de la fonction
%                  assemblage) pour des pml rectangulaires
%   mplo_moyen : mailles par longueur d'onde en moyenne (valeur effective)
function [p,t,e,bord_observation,points_objet,bb_pml,mplo_moyen,n_x]=maillage_diffracteur(dessin,lambda_par_zone,jiggle,mailles_par_longueur_d_onde,nombre_d_onde,fichier_geometrie,indices,rayons)
global l_p_z globalk bords_observation bords_debut_pml bords_dirichlet bords_objets zones_a_indice nombre_d_indices rayon_r1 rayon_r2;
%% diminutifs et globales
rayon_r1=rayons(1);
rayon_r2=rayons(2);
l_p_z=lambda_par_zone;
k=nombre_d_onde;
globalk=k;
lambda=2*pi/k;
nombre_d_indices=max(size(indices));
h=lambda/(mailles_par_longueur_d_onde);
%% maillage
if dessin
    figure;
    pdegplot(fichier_geometrie);
    title('Geometrie');
end
disp('Generation du maillage');
[p,e,t]=initmesh(fichier_geometrie,'Hmax',h,'Jiggle','on');

mplo_moyen=lambda/h;

if dessin
    figure;
    pdemesh(p,e,t);
    title('Maillage');
end
n_x=ones(1,size(t,2));
%% bords
ebo=[];
for bk=1:size(bords_observation,2)
    ebo=[ebo,e(1:2,e(5,:)==bords_observation(bk))]; %#ok<AGROW>
end
bord_observation=zeros(1,size(ebo,2));
bord_observation(1)=ebo(1,1);
for bk=2:(size(ebo,2)+1)
    bord_observation(bk)=ebo(2,ebo(1,:)==bord_observation(bk-1));
end
bpml=[];
for bk=1:size(bords_debut_pml,2)
    bpml=[bpml,e(1,e(5,:)==bords_debut_pml(bk))]; %#ok<AGROW>
end


%% indices et delimitation des pml
if size(zones_a_indice,2)~=size(indices,2)
    error('le nombre d''indices ne correspond pas au nombre de zones a indicer');
end
for ik=1:size(zones_a_indice,2)
    n_x(t(4,:)==zones_a_indice(ik))=indices(ik);
end
bb_pml=zeros(4,1);
bb_pml(1)=min(p(1,bpml));
bb_pml(2)=max(p(1,bpml));
bb_pml(3)=min(p(2,bpml));
bb_pml(4)=max(p(2,bpml));


%% points de comparaison
lpo=[];
for bk=1:size(bords_objets,2)
    lpo=[lpo,e(1:2,e(5,:)==bords_objets(bk))]; %#ok<AGROW>
end
points_objet=p(:,lpo(1,:));
t=t(1:3,:);