% physique.nombreOnde = k
% physique.anglesEmissions = liste d'angles d'ondes planes
%
% U = us+ui avec us la solution  de (Delta +k^2n)us=-k^2(n-1)ui evaluee aux
% points p, ui etant une liste d'ondes planes
function U = Helmholtz( physique, maillage )
%% matrices elements finis telles que (Mk2n-R)*(u_s + ui) = (Mk2n-R-Mq)*ui
[ Mk2n, Mq, R ] = assemblage( maillage, physique );

%% Factorisation LU
try
    if  verLessThan('matlab', '7.4')
        [ LM, UM, PM, QM ] = lu( Mk2n-R );
        RM=[];
    else
       [ LM, UM, PM, QM, RM ] = lu( Mk2n-R );
    end
catch  %#ok<CTCH>
    warning('LU rate : lu incomplet')
    [ LM, UM ] = luinc( Mk2n-R, 1e-2 );
    PM = Mk2n-R;
    QM = []; RM=[];
end


%% Resolution du systeme 
ui = (Mk2n-R-Mq)*exp( +1i*physique.nombreOnde* ...
    maillage.points'*[cos( physique.anglesEmissions ) ; sin( ...
    physique.anglesEmissions ) ] );
if isempty( QM ) %on a recu un lu incomplet : gmres
    U = zeros( size(ui) );
    for ii = 1:size( ui, 2 )
        [ U(:,ii), flag ] = gmres( PM, ui(:,ii), 8, 1e-6, 3, LM, UM );
    end
elseif isempty( RM )
    U = QM*(UM\(LM\(PM*ui))); %cette version de matlab n'a pas donne RM
else
    U = QM*(UM\(LM\(PM*(RM\ui)))); %la totale
end
%%
end