function [ Mk2n, Mq, R ] = assemblage( maillage, physique, indice )
if nargin==2; indice = maillage.indice; end
lambda = 2*pi/physique.nombreOnde;
Epsilon = lambda;
alpha = -3*log( 1e-4 )/Epsilon^3; %parametre de décroissance dans les pml
nTriangles = size( maillage.triangles, 2 );
nPoints = size( maillage.points, 2 );
%disp('Assemblage des matrices')

%% Aires des maillage.triangles
T = aires_triangles( maillage.points, maillage.triangles ).';
%% Gradients
%DET=(x1.*y2)-(x1.*y3)-(x2.*y1)+(x2.*y3)+(x3.*y1)-(x3.*y2);
gradFi1 = [ (maillage.points(2,maillage.triangles(2,:))-...
            maillage.points(2,maillage.triangles(3,:))) ;...
            (-maillage.points(1,maillage.triangles(2,:))+...
            maillage.points(1,maillage.triangles(3,:))) ];
gradFi2 = [ (-maillage.points(2,maillage.triangles(1,:))+...
            maillage.points(2,maillage.triangles(3,:))) ;...
            (maillage.points(1,maillage.triangles(1,:))-...
            maillage.points(1,maillage.triangles(3,:))) ];
gradFi3 = [ (maillage.points(2,maillage.triangles(1,:))-...
            maillage.points(2,maillage.triangles(2,:))) ;...
            (-maillage.points(1,maillage.triangles(1,:))+...
            maillage.points(1,maillage.triangles(2,:))) ];

%% Pml
B =  ([ maillage.points(1,maillage.triangles(1,:)) ;...
        maillage.points(2,maillage.triangles(1,:)) ]+...
      [ maillage.points(1,maillage.triangles(2,:)) ;...
        maillage.points(2,maillage.triangles(2,:)) ]+...
      [ maillage.points(1,maillage.triangles(3,:)) ;...
        maillage.points(2,maillage.triangles(3,:)) ])/3;
sx = ones(1,nTriangles);
sy = ones(1,nTriangles);

sx(B(1,:)<maillage.boitePML(1)) = 1+1i*alpha/physique.nombreOnde*(B(1,B(1,:)<...
                                  maillage.boitePML(1))-maillage.boitePML(1)).^2;
sx(B(1,:)>maillage.boitePML(2)) = 1+1i*alpha/physique.nombreOnde*(B(1,B(1,:)>...
                                  maillage.boitePML(2))-maillage.boitePML(2)).^2;

sy(B(2,:)<maillage.boitePML(3)) = 1+1i*alpha/physique.nombreOnde*(B(2,B(2,:)<...
                                  maillage.boitePML(3))-maillage.boitePML(3)).^2;
sy(B(2,:)>maillage.boitePML(4)) = 1+1i*alpha/physique.nombreOnde*(B(2,B(2,:)>...
                                  maillage.boitePML(4))-maillage.boitePML(4)).^2;

%% Assemblage des matrices
k2n = physique.nombreOnde^2*indice.*sx.*sy.*T/12;
Mk2n = ...
     sparse(maillage.triangles(1,:), maillage.triangles(2,:), k2n, nPoints,...
            nPoints, nTriangles)+sparse(maillage.triangles(1,:),...
            maillage.triangles(3,:), k2n, nPoints, nPoints, nTriangles)...
    +sparse(maillage.triangles(2,:), maillage.triangles(3,:), k2n, nPoints,...
            nPoints, nTriangles)+sparse(maillage.triangles(1,:),...
            maillage.triangles(1,:), k2n, nPoints, nPoints, nTriangles)...
    +sparse(maillage.triangles(2,:), maillage.triangles(2,:), k2n, nPoints,...
            nPoints, nTriangles)+sparse(maillage.triangles(3,:),...
            maillage.triangles(3,:), k2n, nPoints, nPoints, nTriangles);
Mk2n = Mk2n+Mk2n.';

q = physique.nombreOnde^2*(indice-1).*T/12 ;
Mq = ...
    sparse(maillage.triangles(1,:), maillage.triangles(2,:), q, nPoints,...
           nPoints, nTriangles)+sparse(maillage.triangles(1,:),...
           maillage.triangles(3,:), q, nPoints, nPoints, nTriangles)...
   +sparse(maillage.triangles(2,:), maillage.triangles(3,:), q, nPoints,...
           nPoints, nTriangles)+sparse(maillage.triangles(1,:),...
           maillage.triangles(1,:), q, nPoints, nPoints, nTriangles)...
   +sparse(maillage.triangles(2,:), maillage.triangles(2,:), q, nPoints,...
           nPoints, nTriangles)+sparse(maillage.triangles(3,:),...
           maillage.triangles(3,:), q, nPoints, nPoints, nTriangles);
Mq = Mq+Mq.';

R = ...
     sparse(maillage.triangles(1,:), maillage.triangles(2,:),...
            (gradFi1(1,:).*gradFi2(1,:).*sy./sx +...
            gradFi1(2,:).*gradFi2(2,:).*sx./sy)./(4*T), nPoints, nPoints,...
            nTriangles)...
    +sparse(maillage.triangles(1,:), maillage.triangles(3,:),...
            (gradFi1(1,:).*gradFi3(1,:).*sy./sx +...
            gradFi1(2,:).*gradFi3(2,:).*sx./sy)./(4*T), nPoints, nPoints,...
            nTriangles)...
    +sparse(maillage.triangles(2,:), maillage.triangles(3,:),...
            (gradFi2(1,:).*gradFi3(1,:).*sy./sx +...
            gradFi2(2,:).*gradFi3(2,:).*sx./sy)./(4*T), nPoints, nPoints,...
            nTriangles)...
    +sparse(maillage.triangles(1,:), maillage.triangles(1,:),...
            (gradFi1(1,:).*gradFi1(1,:).*sy./sx +...
            gradFi1(2,:).*gradFi1(2,:).*sx./sy)./(8*T), nPoints, nPoints,...
            nTriangles)...%on met la moitié pour pouvoir faire R=R+R.'
    +sparse(maillage.triangles(2,:), maillage.triangles(2,:),...
            (gradFi2(1,:).*gradFi2(1,:).*sy./sx +...
            gradFi2(2,:).*gradFi2(2,:).*sx./sy)./(8*T), nPoints, nPoints,...
            nTriangles)...
    +sparse(maillage.triangles(3,:), maillage.triangles(3,:),...
            (gradFi3(1,:).*gradFi3(1,:).*sy./sx +...
            gradFi3(2,:).*gradFi3(2,:).*sx./sy)./(8*T), nPoints, nPoints,...
            nTriangles);
R = R+R.';
end